@@ -6,18 +6,58 @@
 import math
 import queue
 import logging
+import time
 from . import mqtt_interface
 from . import file_interface
 
+# -------------------------------------------------------------------------
+# Global variables used within the Common Module
+# -------------------------------------------------------------------------
+
+# Global variables for how the signals/points/sections buttons appear
+# on the screen. This is to allow the appearance to be optimised for
+# particular window sizes/screen resolutions.
+fontsize = 9  # Used by the Signals, Points and sections modules
+xpadding = 4  # Used by the Signals, Points and sections modules
+ypadding = 3  # Used by the Signals, Points and sections modules
+bgraised = "grey85"   # Used by the Signals and Points modules
+bgsunken = "white"    # Used by the Signals and Points modules
+
+# Global Variable to hold a reference to the TkInter Root Window
+root_window = None
+# Event queue for passing "commands" back into the main tkinter thread
+event_queue = queue.Queue()
+# Global variable to signal (to other modules) that application is closing
+shutdown_initiated = False
+
 #-------------------------------------------------------------------------
 # Function to catch the root window close event so we can perform an
 # orderly shutdown of the other threads running in the application
+# The shutdown_initiated flag is used to tell the flash aspects and timed
+# signals functions to stop scheduling further "after" commands and exit 
 #-------------------------------------------------------------------------
 
 def on_closing():
-    global root_window
+    global logging
+    global shutdown_initiated
     if file_interface.save_state_and_quit():
+        logging.info ("Initiating Shutdown")
+        shutdown_initiated = True
+        # Clear out any retained messages and disconnect from broker
         mqtt_interface.mqtt_shutdown()
+        # Wait until all the tasks we have scheduled via the tkinter 'after' method have completed
+        # We need to put a timeout around this to deal with any ongoing timed signal sequences
+        # (although its unlikely the user would initiate a shut down until these have finished)
+        timeout_start = time.time()
+        while time.time() < timeout_start + 30:
+            if root_window.tk.call('after','info') != "":
+                root_window.update()
+                time.sleep(0.01)
+            else:
+                logging.info ("Exiting Application")
+                break
+        if time.time() >= timeout_start + 30:
+            logging.warning ("Timeout waiting for scheduled tkinter events to complete - Exiting anyway")
         root_window.destroy()
     return()
 
@@ -30,9 +70,6 @@ def on_closing():
 # objects should be done from within the main tkinter thread.
 #-------------------------------------------------------------------------
 
-root_window = None
-event_queue = queue.Queue()
-
 def find_root_window (canvas):
     global root_window
     parent = canvas.master
@@ -66,6 +103,7 @@ def handle_callback_in_tkinter_thread(*args):
     return()
     
 def execute_function_in_tkinter_thread(callback_function):
+    global logging
     callback = event_queue.put(callback_function)
     if root_window is not None:
         root_window.event_generate("<<ExtCallback>>", when="tail")
@@ -73,18 +111,6 @@ def execute_function_in_tkinter_thread(callback_function):
         logging.error ("execute_function_in_tkinter_thread - cannot execute callback function as root window is undefined")
     return()
 
-# -------------------------------------------------------------------------
-# Global variables for how the signals/points/sections buttons appear
-# on the screen. This is to allow the appearance to be optimised for
-# particular window sizes/screen resolutions.
-# -------------------------------------------------------------------------
-
-fontsize = 9  # Used by the Signals, Points and sections modules
-xpadding = 4  # Used by the Signals, Points and sections modules
-ypadding = 3  # Used by the Signals, Points and sections modules
-bgraised = "grey85"   # Used by the Signals and Points modules
-bgsunken = "white"    # Used by the Signals and Points modules
-
 # -------------------------------------------------------------------------
 # Common functions to rotate offset coordinates around an origin
 # The angle should be passed into these functions in degrees.
