@@ -2,14 +2,18 @@
 # This enables the current configuration of the signals, points and sections on the layout to be
 # "saved" when the application is closed and then "loaded" when the application is re-loaded
 # (ready for the next running session)
+# ----------------------------------------------------------------------------------------------
+# 
+# Public Types and Functions:
 # 
 # load_layout_state - Loads the initial state for all 'points', 'signals' and 'sections' from file
 #                     and enables the save of the current layout state to file on application quit.
-#                     If load is "cancelled" or "file not found" then the default state will be used
+#                     If load is "cancelled" or "file not found" then the default state will be used.
 #    Optional Parameters:
 #       file_name:str - to load/save - default = None (will default to 'main-python-script.sig')
 #       load_file_dialog:bool - Opens a 'load file' dialog to select a file - default = False
-#       save_file_dialog:bool - Opens a 'save file' dialog on application quit - default = False#
+#       save_file_dialog:bool - Opens a 'save file' dialog on application quit - default = False
+#
 #------------------------------------------------------------------------------------------------
 
 import os
@@ -20,6 +24,7 @@ import tkinter.messagebox
 import tkinter.filedialog
 from . import signals_common
 from . import track_sections
+from . import block_instruments
 from . import points
 
 #-------------------------------------------------------------------------------------------------
@@ -35,6 +40,36 @@ save_as_option_enabled = True
 
 layout_state ={}
 
+#-------------------------------------------------------------------------------------------------
+# Define all the layout elements that needs to be saved/loaded for each module. This effectively
+# makes the remainder of the code generic. If we ever need to add another "layout_element" or
+# another "item_element" (to an existing layout element) then we just define that here
+# Note that we also define the type for each element to enable a level of validation on load
+# This is also important for "enum" types where we have to save the VALUE of the Enum
+#-------------------------------------------------------------------------------------------------
+
+def get_sig_file_config(get_sig_file_data:bool = False):
+    
+    signal_elements = ( ("sigclear","bool"),("subclear","bool"),("override","bool"),
+                        ("siglocked","bool") ,("sublocked","bool"),("routeset","enum"),
+                        ("releaseonred","bool"),("releaseonyel","bool"),("theatretext","str") )
+    point_elements = ( ("switched","bool"),("fpllock","bool"),("locked","bool") )
+    section_elements = ( ("occupied","bool"),("labeltext","str") )
+    instrument_elements = ( ("sectionstate","bool"),("repeaterstate","bool") )
+
+    layout_elements = { "signals"    : {"elements" : signal_elements},
+                        "points"     : {"elements" : point_elements},
+                        "sections"   : {"elements" : section_elements},
+                        "instruments": {"elements" : instrument_elements} }
+    
+    if get_sig_file_data:
+        layout_elements["points"]["source"] = points.points
+        layout_elements["signals"]["source"] = signals_common.signals
+        layout_elements["sections"]["source"] = track_sections.sections
+        layout_elements["instruments"]["source"] = block_instruments.instruments
+        
+    return(layout_elements)
+    
 #-------------------------------------------------------------------------------------------------
 # Public API function to load the initial layout state from File (and also configure what options
 # are presented to the user on quit of the application (i.e. options for saving layout state)
@@ -120,7 +155,7 @@ def load_layout_state(file_name:str=None,
 #
 # If the filename is 'None' then 'load_layout_state' was never called (i.e. the signalling application
 # isn't using this feature). In this case we just provide the user with an option to quit or cancel.
-# If the filename is defined then we provide the user with an option to save (yes/no/cancel) and
+# If the filename is not 'None' then we provide the user with an option to save (yes/no/cancel) and
 # use the filename for the file that was loaded (or attempted) by the 'load_layout_state' function.
 # If 'save_as_option_enabled' is True, then we open a file dialogue to select a file.
 #
@@ -162,71 +197,46 @@ def save_state_and_quit():
                 # Note that the asksaveasfilename dialog returns the fully qualified filename
                 # (including the path) - we only need the name so strip out the path element
                 # This also makes it clearer to see the default filename in the file save dialog
-                if filename != () and filename != "":
-                    path,name = os.path.split(filename)
-                    filename = name            
+                path,name = os.path.split(filename)
+                filename = name            
                 logging.info("Saving Layout State Information as '"+filename+"'")
-                # Compile a dictionary of everything we want to save. We test each element
-                # before trying to add it to the dictionary to handle remote signals
-                signal_states = {}
-                for signal in signals_common.signals:
-                    signal_states[signal] ={}
-                    if "sigclear" in signals_common.signals[signal].keys(): 
-                        signal_states[signal]["sigclear"] = signals_common.signals[signal]["sigclear"]
-                    else: 
-                        signal_states[signal]["sigclear"] = None
-                    if "subclear" in signals_common.signals[signal].keys(): 
-                        signal_states[signal]["subclear"] = signals_common.signals[signal]["subclear"]
-                    else:
-                        signal_states[signal]["subclear"] = None
-                    if "override" in signals_common.signals[signal].keys(): 
-                        signal_states[signal]["override"] = signals_common.signals[signal]["override"]
-                    else:
-                        signal_states[signal]["override"] = None
-                    if "siglocked" in signals_common.signals[signal].keys(): 
-                        signal_states[signal]["siglocked"] = signals_common.signals[signal]["siglocked"]
-                    else:
-                        signal_states[signal]["siglocked"] = None
-                    if "sublocked" in signals_common.signals[signal].keys(): 
-                        signal_states[signal]["sublocked"] = signals_common.signals[signal]["sublocked"]
-                    else:
-                        signal_states[signal]["sublocked"] = None
-                    if "routeset" in signals_common.signals[signal].keys(): 
-                        # Use the 'value' for  Enumeration types as the enumerations can't be converted
-                        signal_states[signal]["routeset"] = signals_common.signals[signal]["routeset"].value
-                    else:
-                        signal_states[signal]["routeset"] = None
-                    if "releaseonred" in signals_common.signals[signal].keys(): 
-                        signal_states[signal]["releaseonred"] = signals_common.signals[signal]["releaseonred"]
-                    else:
-                        signal_states[signal]["releaseonred"] = None
-                    if "releaseonyel" in signals_common.signals[signal].keys(): 
-                        signal_states[signal]["releaseonyel"] = signals_common.signals[signal]["releaseonyel"]
-                    else:
-                        signal_states[signal]["releaseonyel"] = None
-                    if "theatretext" in signals_common.signals[signal].keys(): 
-                        signal_states[signal]["theatretext"] = signals_common.signals[signal]["theatretext"]
-                    else:
-                        signal_states[signal]["theatretext"] = None
-                point_states = {}
-                for point in points.points:
-                    point_states[point] ={}
-                    point_states[point]["switched"] = points.points[point]["switched"]
-                    point_states[point]["fpllock"] = points.points[point]["fpllock"]
-                    point_states[point]["locked"] = points.points[point]["locked"]
-                section_states = {}
-                for section in track_sections.sections:
-                    section_states[section] ={}
-                    section_states[section]["occupied"] = track_sections.sections[section]["occupied"]
-                    section_states[section]["labeltext"] = track_sections.sections[section]["labeltext"]
-                layout_state = {}
-                layout_state["info"] = "Model Railway Signalling State File"
-                layout_state["signals"] = signal_states
-                layout_state["points"] = point_states
-                layout_state["sections"] = section_states
-                # convert the file to a human readable json format
-                file_contents = json.dumps(layout_state,indent=4,sort_keys=True)
-                # save the file
+                # Create a dictionary to hold the data we need to save 
+                dictionary_to_save ={"info": "Model Railway Signalling State File"}
+                # Retrieve the DEFINITION of all the data items we need to save to maintain state
+                # These are defined in a single function at the top of this source file. We also
+                # retrieve the source DATA we need to save from the various source dictionaries
+                layout_elements = get_sig_file_config(get_sig_file_data=True)
+                # Iterate through the main LAYOUT-ELEMENTS (e.g. signals, points, sections etc)
+                for layout_element in layout_elements:
+                    # For each LAYOUT ELEMENT create a sub-dictionary to hold the individual ITEMS
+                    # The individual ITEMS will be the individual points, signals, sections etc
+                    dictionary_to_save[layout_element] = {}
+                    # Get the dictionary containing the source data for this LAYOUT ELEMENT
+                    source_data_dictionary = layout_elements[layout_element]["source"]
+                    # Get the list of the ITEM ELEMENTS we need to save for this LAYOUT ELEMENT
+                    item_elements_to_save = layout_elements[layout_element]["elements"]
+                    # Iterate through the ITEMS that exist for this LAYOUT ELEMENT
+                    # Each ITEM represents a specific signal, point, section etc
+                    for item in source_data_dictionary:
+                        # For each ITEM, create a sub-dictionary to hold the individual ITEM ELEMENTS
+                        # Each ITEM ELEMENT represents a specific parameter for an ITEM (e.g."sigclear") 
+                        dictionary_to_save[layout_element][item] = {}
+                        # Iterate through the ITEM ELEMENTS to save for the specific ITEM
+                        for item_element in item_elements_to_save:
+                            # Value [0] is the element name, Value [1] is the element type
+                            if item_element[0] not in source_data_dictionary[item].keys():
+                                # if the element isn't present in the source dict then we save a NULL value
+                               dictionary_to_save[layout_element][item][item_element[0]] = None
+                            elif item_element[1]=="enum":
+                                # Enumeration values cannot be converted to json as is - we need to use the value
+                                parameter = source_data_dictionary[item][item_element[0]]
+                                dictionary_to_save[layout_element][item][item_element[0]] = parameter.value
+                            else:
+                                # The Json conversion should support all standard python types
+                                parameter = source_data_dictionary[item][item_element[0]]
+                                dictionary_to_save[layout_element][item][item_element[0]] = parameter
+                # convert the file to a human readable json format and save the file
+                file_contents = json.dumps(dictionary_to_save,indent=4,sort_keys=True)
                 try:
                     with open (filename,'w') as file:
                         file.write(file_contents)
@@ -236,132 +246,64 @@ def save_state_and_quit():
     return (quit_application)
 
 #-------------------------------------------------------------------------------------------------
-# Function called on creation of a track section object to return the initial state from the loaded
-# layout state. If no layout state has been loaded or the loaded data doesn't include an entry for
-# the section then we return 'None' and the section will retain its "as created" default state
+# Function called on creation of a signal/point/section/instrument Object to return the initial state
+# from the loaded data. If no layout state has been loaded or the loaded data doesn't include an
+# entry for the Object then we return 'None' and the Object will retain its "as created" default state
 #-------------------------------------------------------------------------------------------------
 
-def get_initial_section_state(section_id):
+def get_initial_item_state(layout_element:str,item_id:int):
     global logging
-    section_state={}
-    if "sections" not in layout_state.keys():
-        # This could be a valid condition if no file has been loaded - we therefore fail silently
-        section_state["occupied"] = None
-        section_state["labeltext"] = None
-    elif str(section_id) not in layout_state["sections"].keys():
-        # We know a file is loaded - therefore this is a valid error to report
-        logging.warning("Section "+str(section_id)+": Loaded file missing data for section - Default values will be set")
-        section_state["occupied"] = None
-        section_state["labeltext"] = None
+    # Retrieve the DEFINITION of all the data items that are available
+    sig_file_config = get_sig_file_config()
+    # Check if the requested LAYOUT ELEMENT is a supported
+    if layout_element not in sig_file_config.keys():
+        logging.error("File Interface - Item type not supported : "+layout_element)
+        state_to_return = None
     else:
-        # We use exception handling in case the data elements are missing or corrupted (i.e. the wrong type)
-        try:
-            section_state["occupied"] = bool(layout_state["sections"][str(section_id)]["occupied"])
-            section_state["labeltext"] = str(layout_state["sections"][str(section_id)]["labeltext"])
-        except Exception as exception:
-            logging.error("Section "+str(section_id)+": Loaded file data elements corrupted - Default values will be set")
-            logging.error("Section "+str(section_id)+": Reported exception: "+str(exception))
-            section_state["occupied"] = None
-            section_state["labeltext"] = None
+        # Create a dictionary to hold the state information we want to return
+        state_to_return = {}
+        # Iterate through the ITEM ELEMENTS we are interested in for the LAYOUT ELEMENT and
+        # set an initial value of NONE (to be returned if we fail to validate the loaded data)
+        for item_element in sig_file_config[layout_element]["elements"]:
+            # retrieve the required ITEM ELEMENT Name
+            item_element_name = item_element[0]
+            state_to_return[item_element_name] = None
+        # See if the specified LAYOUT ELEMENT exists in the loaded file 
+        if not layout_element in layout_state.keys():
+            # This could be a valid condition if no file has been loaded - fail silently
+            pass
+        # See if the specified ITEM (for the LAYOUT ELEMENT) exists in the loaded file
+        elif str(item_id) not in layout_state[layout_element].keys():
+            # We know a file is loaded - therefore this is a valid error to report
+            logging.warning("File Interface - Data missing for '"+layout_element+"-"
+                                                +str(item_id)+"' - Default values will be set")
         else:
-            logging.info("Section "+str(section_id)+": Successfully loaded initial state")
-    return(section_state)
+            # Iterate through the ITEM ELEMENTS we are interested in for the LAYOUT ELEMENT
+            for item_element in sig_file_config[layout_element]["elements"]:
+                # retrieve the required ITEM ELEMENT Name and expected ITEM ELEMENT Type
+                element_name = item_element[0]
+                element_type = item_element[1]
+                # Test to see if the required ITEM ELEMENT is present for the ITEM
+                if element_name not in layout_state[layout_element][str(item_id)]:
+                    logging.warning("File Interface - Data missing for '"+layout_element +"-"
+                            +str(item_id)+"-"+element_name+"' - Default value will be set")
+                else:
+                    # Retrieve the ITEM ELEMENT Value from the loaded data
+                    element_value = layout_state[layout_element][str(item_id)][element_name]
+                    # We can do some basic validation on the loaded data to check the expected type 
+                    if element_type == "bool" and not isinstance(element_value,bool) and element_value is not None:
+                        logging.warning("File Interface - Data corrupted for '"+layout_element
+                                +"-"+str(item_id)+"-"+element_name+"' - Default value will be set")
+                    elif element_type == "str" and not isinstance(element_value,str) and element_value is not None: 
+                        logging.warning("File Interface - Data corrupted for '"+layout_element
+                                +"-"+str(item_id)+"-"+element_name+"' - Default value will be set")
+                    elif element_type == "enum" and not isinstance(element_value,int) and element_value is not None:
+                        logging.warning("File Interface - Data corrupted for '"+layout_element
+                                +"-"+str(item_id)+"-"+element_name+"' - Default value will be set")
+                    else:
+                        # Add the ITEM ELEMENT (and the loaded ITEM VALUE) to the dictionary
+                        state_to_return[element_name] = element_value
 
-#-------------------------------------------------------------------------------------------------
-# Function called on creation of a point object to return the initial state from the loaded
-# layout state. If no layout state has been loaded or the loaded data doesn't include an entry for
-# the point then we return 'None' and the point will retain its "as created" default state
-#-------------------------------------------------------------------------------------------------
-
-def get_initial_point_state(point_id):
-    global logging
-    point_state={}
-    if "points" not in layout_state.keys():
-        # This could be a valid condition if no file has been loaded - we therefore fail silently
-        point_state["switched"] = None
-        point_state["fpllock"] = None
-        point_state["locked"] = None
-    elif str(point_id) not in layout_state["points"].keys():
-        # We know a file is loaded - therefore this is a valid error to report
-        logging.warning("Point "+str(point_id)+": Loaded file missing data for point - Default values will be set")
-        point_state["switched"] = None
-        point_state["fpllock"] = None
-        point_state["locked"] = None
-    else:
-        # We use exception handling in case the data elements are missing or corrupted (i.e. the wrong type)
-        try:
-            point_state["switched"] = bool(layout_state["points"][str(point_id)]["switched"])
-            point_state["fpllock"] = bool(layout_state["points"][str(point_id)]["fpllock"])
-            point_state["locked"] = bool(layout_state["points"][str(point_id)]["locked"])
-        except Exception as exception:
-            logging.error("Point "+str(point_id)+": Loaded file data elements corrupted - Default values will be set")
-            logging.error("Point "+str(point_id)+": Reported exception: "+str(exception))
-            point_state["switched"] = None
-            point_state["fpllock"] = None
-            point_state["locked"] = None
-        else:
-            logging.info("Point "+str(point_id)+": Successfully loaded initial state")
-    return(point_state)
-
-#-------------------------------------------------------------------------------------------------
-# Function called on creation of a signal object to return the initial state from the loaded
-# layout state. If no layout state has been loaded or the loaded data doesn't include an entry for
-# the signal then we return 'None' and the signal will retain its "as created" default state
-#-------------------------------------------------------------------------------------------------
-
-def get_initial_signal_state(sig_id):
-    global logging
-    signal_state={}
-    if "signals" not in layout_state.keys():
-        # This could be a valid condition if no file has been loaded - we therefore fail silently
-        signal_state["sigclear"] = None
-        signal_state["subclear"] = None
-        signal_state["override"] = None
-        signal_state["siglocked"] = None
-        signal_state["sublocked"] = None
-        signal_state["releaseonred"] = None
-        signal_state["releaseonyel"] = None
-        signal_state["theatretext"] = None
-        signal_state["routeset"] = None
-    elif str(sig_id) not in layout_state["signals"].keys():
-        # We know a file is loaded - therefore this is a valid error to report
-        logging.warning("Signal "+str(sig_id)+": Loaded file missing data for signal - Default values will be set")
-        signal_state["sigclear"] = None
-        signal_state["subclear"] = None
-        signal_state["override"] = None
-        signal_state["siglocked"] = None
-        signal_state["sublocked"] = None
-        signal_state["releaseonred"] = None
-        signal_state["releaseonyel"] = None
-        signal_state["theatretext"] = None
-        signal_state["routeset"] = None
-    else:
-        # We use exception handling in case the data elements are missing or corrupted (i.e. the wrong type)
-        try:
-            signal_state["sigclear"] = bool(layout_state["signals"][str(sig_id)]["sigclear"])
-            signal_state["subclear"] = bool(layout_state["signals"][str(sig_id)]["subclear"])
-            signal_state["override"] = bool(layout_state["signals"][str(sig_id)]["override"])
-            signal_state["siglocked"] = bool(layout_state["signals"][str(sig_id)]["siglocked"])
-            signal_state["sublocked"] = bool(layout_state["signals"][str(sig_id)]["sublocked"])
-            signal_state["releaseonred"] = bool(layout_state["signals"][str(sig_id)]["releaseonred"])
-            signal_state["releaseonyel"] = bool(layout_state["signals"][str(sig_id)]["releaseonyel"])
-            signal_state["theatretext"] = str(layout_state["signals"][str(sig_id)]["theatretext"])
-            # We load the 'values' of enumeration types - so need to convert them back to the enumerations
-            signal_state["routeset"] = signals_common.route_type(layout_state["signals"][str(sig_id)]["routeset"])
-        except Exception as exception:
-            logging.error("Signal "+str(sig_id)+": Loaded file data elements corrupted - Default values will be set")
-            logging.error("Signal "+str(sig_id)+": Reported exception: "+str(exception))
-            signal_state["sigclear"] = None
-            signal_state["subclear"] = None
-            signal_state["override"] = None
-            signal_state["siglocked"] = None
-            signal_state["sublocked"] = None
-            signal_state["releaseonred"] = None
-            signal_state["releaseonyel"] = None
-            signal_state["theatretext"] = None
-            signal_state["routeset"] = None
-        else:
-            logging.info("Signal "+str(sig_id)+": Successfully loaded initial state ")
-    return(signal_state)
+    return(state_to_return)
 
 ############################################################################################################
\ No newline at end of file
